(*  Title:      NSPKclassic.thy
    SPDX-License-Identifier: BSD-3-Clause
*)

section\<open>NSPK Classic\<close>
theory NSPKclassic
  imports "Automated_Stateful_Protocol_Verification.PSPSP"
begin

declare [[pspsp_timing]]

trac\<open>
Protocol: nspkclassic

Enumerations:
honest = {a,b}
dishon = {i}
agent = honest ++ dishon

Sets:
challenged/2
responded/2

Functions:

Public crypt/2 step1format/2 step2format/2 pk/1 nonce/1
Private inv/1

Analysis:

step1format(X,Y) -> X,Y
step2format(X,Y) -> X,Y
crypt(X,Y) ? inv(X) -> Y
nonce(X) -> X

Transactions:

a1b(A: dishon)
send inv(pk(A)).

a6()
new NA
send NA.

step1(A: honest, B: agent)
new NA
insert NA challenged(A,B)
send crypt(pk(B),step1format(NA,A)).

step2(B: honest, A: agent, NA: value)
receive crypt(pk(B),step1format(NA,A))
new NB
insert NB challenged(B,A)
send crypt(pk(A),step2format(NA,NB)).

step3(A: honest, B: agent, NA: value, NB: value)
receive crypt(pk(A),step2format(NA,NB))
NA in challenged(A,B)
insert NA responded(A,B)
delete NA challenged(A,B)
send crypt(pk(B),nonce(NB)).

step3receive(B: honest, A: agent, NB: value)
receive crypt(pk(B),nonce(NB))
NB in challenged(B,A)
delete NB challenged(B,A)
insert NB responded(B,A).

att1(A: honest, B: honest, NA: value)
receive NA
NA in challenged(A,B)
attack.

att2(A: honest, B: honest, NA: value)
receive NA
NA in responded(A,B)
attack.

\<close>

subsection \<open>Protocol model setup\<close>
protocol_model_setup spm: nspkclassic


subsection \<open>Fixpoint computation\<close>
compute_fixpoint nspkclassic_protocol nspkclassic_fixpoint
value "let (FP,_,TI) = nspkclassic_fixpoint in (size FP, size TI)"


\<comment> \<open>The fixpoint contains an attack signal\<close>
value "attack_notin_fixpoint nspkclassic_fixpoint"


subsection \<open>The generated theorems and definitions\<close>

thm nspkclassic_enum_consts.nchotomy
thm nspkclassic_sets.nchotomy
thm nspkclassic_fun.nchotomy
thm nspkclassic_atom.nchotomy
thm nspkclassic_arity.simps
thm nspkclassic_public.simps
thm nspkclassic_\<Gamma>.simps
thm nspkclassic_Ana.simps

thm nspkclassic_protocol_def

end
